/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.htmltools;

import java.io.Writer;
import java.io.StringWriter;
import java.io.IOException;
import java.io.FilenameFilter;
import java.io.File;
import java.net.URL;
import java.awt.*;
import java.awt.event.*;
import de.rw7.token.FastInput;
import de.rw7.token.HtmlConsumer;
import de.rw7.token.HtmlTokenizer;
import de.rw7.token.TextTokenizer;

public class HtmlToc extends Frame 
                     implements HtmlConsumer, 
                                WindowListener, 
                                ActionListener,
                                ItemListener,
                                FilenameFilter
{
  private String src;
  private TextArea ta;
  private TextField tf;
  private Button file,start;
  private Checkbox cssC,ancC,qryC,glbC;
  
  public HtmlToc(String src)
  {
    super("Html Contents Lister");
    src = (src!=null) ? src : "";
    this.src=src;
    setSize(new Dimension(400,400));
    addWindowListener((WindowListener)this);
    
    Panel P=new Panel(new BorderLayout());
    add(P, BorderLayout.NORTH);
    P.add(tf=new TextField(src), BorderLayout.CENTER);
    P.add(file=new Button(" .. "), BorderLayout.EAST);

    add(ta=new TextArea(), BorderLayout.CENTER);
    
    P=new Panel(new BorderLayout(10,10));
    add(P,"South");
    P.add(start=new Button("  start  "), BorderLayout.WEST);

    Panel Q=new Panel(new GridLayout(0,1));
    P.add(Q, BorderLayout.CENTER);
    Q.add(cssC=new Checkbox("generate css classes",    true ));
    Q.add(ancC=new Checkbox("ignore non-anchored headings",false));
    Q.add(qryC=new Checkbox("generate querys", true ));
    Q.add(glbC=new Checkbox("generate global tags", true ));

    updateArea();
    tf.addActionListener(this);
    file.addActionListener(this);
    start.addActionListener(this);
    cssC.addItemListener(this);
    ancC.addItemListener(this);
    qryC.addItemListener(this);
    glbC.addItemListener(this);    
  }
  
  public boolean accept(File dir, String name)
  {
    System.err.println(name);
    return(
      name.endsWith(".html")||
      name.endsWith(".shtml")||
      name.endsWith(".htm"));
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource()==file)
    {
      try
      {
        FileDialog fd=new FileDialog(this, "Select Source HTML");
        fd.setFilenameFilter(this);
        fd.setFile("*.html;*.htm");
        fd.show();
        if(fd.getFile()!=null)
        {
          src="file:///"+fd.getDirectory()+fd.getFile();
          tf.setText(src);
          updateArea();
        }
      }
      catch(java.lang.SecurityException se)
      {
        (new MessageDialog(this, "Security Violation", 
          "Security violation while opening system file dialog:\n\n"+
          se.toString()+
          "\n\nPlease check permissions in the browser / JVM."
        )).show();
      }
    }
    else
    {
      src=tf.getText();
      updateArea();
    }
  }
  
  public void itemStateChanged(ItemEvent e)
  {
    updateArea();
  }
  
  private boolean css,anc,qry,glb;
  
  private void updateArea()
  {
    if(src.length()==0) { ta.setText("no url specified"); return; }

    css=cssC.getState();
    anc=ancC.getState();
    qry=qryC.getState();
    glb=glbC.getState();
    try
    {
      StringWriter str=new StringWriter();
      read(str);
      str.close();
      ta.setText(str.toString());
    }
    catch(IOException e) { ta.setText(e.toString()); }
    catch(SecurityException e) { ta.setText(e.toString()); }
  }
  

  private HtmlTokenizer ht;
  private TextTokenizer tt;
  private Writer out;
  public void read(Writer out) throws IOException
  {
    if(glb) out.write("<HTML><BODY>\n");
    FastInput r=new FastInput((new URL(src)).openStream());

    ht=new HtmlTokenizer(0);
    ht.addTag("H1",0, null);
    ht.addTag("H2",1, null);
    ht.addTag("H3",2, null);
    ht.addTag("H4",3, null);
    ht.addTag("H5",4, null);
    ht.addTag("H6",5, null);
    String[] attrA = { "NAME" };
    ht.addOpeningTag("A", -2, attrA); //ht.printTree();
    tt=new TextTokenizer();
    tt.ignoreEntitys=true;
    this.out=out;
    try { ht.read(r,this); } catch(FastInput.EndException e){};
    newLevel(-1);
    this.out=null;
    ht=null;

    r.close();
    if(glb) out.write("</BODY></HTML>\n");
  }
  
  
  private int lastH=-1, currH=-1;
  private String name;
  private boolean inH=false;
  
  private final void newLevel(int v)
  {
    currH=v;
    try
    {
      for(int i=lastH; i<currH; i++)
      {
        out.write("<UL");
        if(css) out.write(" CLASS=\"CH"+(i+2)+"\"");
        out.write(">\n");
      }
      for(int i=currH; i<lastH; i++) out.write("</UL>\n");
    }
    catch(IOException e) { System.err.println(e); }
    lastH=currH;
  }

  public int onOpeningTag(int code, String tag, String[] attr)
  {
    //System.err.println("O: "+tag);
    if(0<=code&&code<=5)
    {
      if(attr!=null) throw new IllegalArgumentException();
      name=null;
      newLevel(code);
      inH=true;
      return HtmlTokenizer.MODE_PARSE_TEXT;
    }
    else if(code==-2)
    {
      if(attr.length!=1) throw new IllegalArgumentException();
      name=attr[0];
      return inH ? HtmlTokenizer.MODE_PARSE_TEXT : 0;
    }
    else throw new IllegalArgumentException();
  }
       
  public int onClosingTag(int code, String tag)
  {
    //System.err.println("C: "+tag);
    if(0<=code&&code<=5)
    {
      if(currH!=code) throw new IllegalArgumentException("Heading tags not constistent");
      try
      {
        String s=tt.getText();
        if(!anc || (name!=null))
        {
          out.write("<LI>");
          if(qry&&(name!=null)) { out.write("<A HREF=\"#"); out.write(name); out.write("\">"); }
          out.write(s);
          if(qry&&(name!=null)) out.write("</A>");
          out.write("</LI>\n");
        }
      }
      catch(IOException e) { System.err.println(e); }
      inH=false; 
    }
    else throw new IllegalArgumentException();
    return 0;
  }
  
  public void onText(FastInput in) throws IOException, FastInput.EndException
  {
    tt.read(in);
  }

  public void windowActivated(WindowEvent e){}
  public void windowDeactivated(WindowEvent e){}
  public void windowDeiconified(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowOpened(WindowEvent e){}
  public void windowClosing(WindowEvent e){dispose();}
  public void windowClosed(WindowEvent e){if(only)System.exit(0);}
  private static boolean only=false;
  public static void main(String[] args)
  {
    only=true;
    (new HtmlToc((args.length==0) ? null : args[0])).show();
  }

}
