/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.htmltools;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.awt.*;
import java.awt.event.*;

/**
  This is the main window of the LinkVerify.
*/

public class LinkFrame
  extends Frame
  implements ActionListener,
             WindowListener
{
  protected Button file;
  protected Button start;
  protected Button exit;
  protected TextField root;
  protected Checkbox externals;
  protected int nthreads;

  public LinkFrame(String root,
                   boolean externals,
                   int nthreads)
  {
    super("LinkVerify");
    Label label;
    setLayout(new BorderLayout());
    {
      Panel Q=new Panel(new BorderLayout());
      Q.add(label=new Label("Root URL"), BorderLayout.NORTH);
      Q.add(this.root=new TextField((root!=null)?root:""),BorderLayout.CENTER);
      Q.add(file=new Button(" .. "),BorderLayout.EAST);
      add(Q, BorderLayout.NORTH);
    }
    add(this.externals=new Checkbox("Verify external links", externals), BorderLayout.WEST);
    {
      Panel P=new Panel(new FlowLayout(FlowLayout.RIGHT));
      P.add(start=new Button("Start"));
      P.add(exit=new Button("Exit"));
      add(P, BorderLayout.SOUTH);
    }
    setSize(400,180);
    validate();

    this.nthreads=nthreads;

    addWindowListener((WindowListener)this);
    file.addActionListener(this);
    start.addActionListener(this);
    exit.addActionListener(this);
    this.root.addActionListener(this);
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource()==file)
    {
        try
        {
          FileDialog fd=new FileDialog(this, "Select Root Url");
          fd.show();
          if(fd.getFile()!=null)
            root.setText(fd.getDirectory()+fd.getFile());
          //throw new java.lang.SecurityException("Long Description what happened");
        }
        catch(java.lang.SecurityException se)
        {
          (new MessageDialog(this, "Security Violation",
            "Security violation while \nopening system file dialog:\n\n"+
            se.getMessage()+
            "\n\nPlease check permissions\nin the browser / JVM."
          )).show();
        }
    }
    else if(e.getSource()==start||e.getSource()==root)
    {
      try
      {
        (new LinkResult(LinkVerify.convert(root.getText()),
                        externals.getState(),
                        nthreads)).show();
      }
      catch(java.net.MalformedURLException ex)
      {
        (new MessageDialog(this, "Invalid Root URL","The URL you have entered\nis not valid.\n\n"+ex.getMessage())).show();
      }
    }
    else if(e.getSource()==exit)
    {
      dispose();
    }
    else
    {
      throw new IllegalArgumentException();
    }
  }

  public void windowActivated(WindowEvent e){}
  public void windowDeactivated(WindowEvent e){}
  public void windowDeiconified(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowOpened(WindowEvent e){}
  public void windowClosing(WindowEvent e){dispose();}
  public boolean exitJVMonDispose=false;
  public void windowClosed(WindowEvent e)
  {
    if(exitJVMonDispose) System.exit(0);
  }
}

class LinkResult
  extends Frame 
  implements ActionListener,
             WindowListener,
             LinkFrontend
{
  protected java.net.URL root;
  protected boolean externals;

  /**
    Hold the number of spiders, run parallely when checking
    a website. Its mainly used to configure the LinkSpider.
    @see LinkSpider
  */
  protected int nthreads;
  
  /**
    Holds the Spider when and only when running the spider.
    Is null when not running.
  */
  protected LinkSpider spider;
  
  /**
    A list of the referrers of the currently selected
    broken link / target.
  */
  protected List refL;
  
  protected Button abort;
  protected Button progress;
  protected Button close;

  protected CardLayout chartL;
  protected Container chartCt;

  class Chart implements ActionListener, ItemListener
  {
    String name;

    /**
      A list of the LinkTag-Objects of all broken links / targets,
      parallely to lnkL.
    */
    Vector vector;
    
    List list;

    Chart(Panel chartP, Panel buttonP, String name, String caption)
    {
      this.name=name;
      
      vector=new Vector();

      Button b=new Button(name);
      buttonP.add(b);
      b.addActionListener(this);
      
      Panel panel=new Panel(new BorderLayout());
      panel.add(new Label(caption), BorderLayout.NORTH);      
      panel.add(list=new List(), BorderLayout.CENTER);
      list.addItemListener(this);
      chartP.add(panel, name);
    }
    
    public void actionPerformed(ActionEvent e)
    {
      //System.err.println(e);
      if(list.getSelectedIndex()<0 && list.getItemCount()>0) 
        list.select(0);
      chartL.show(chartCt, name);
      list.requestFocus();
      updateReferrers();
    }
  
    public void itemStateChanged(ItemEvent e)
    {
      //System.err.println(e);
      if(e.getSource()==list)
      {
        updateReferrers();
      }
      else throw new IllegalArgumentException();
    }
    
    public void reset()
    {
      // the getItemCount()>0 is a workaround for a bug in netscape4-java
      if(list.getItemCount()>0) list.removeAll(); 
      vector.removeAllElements(); 
    }

    /**
      Is called, whenever the currently selected broken link
      has changed, and an update of the referrers list is needed.
    */
    void updateReferrers()
    {
      // the getItemCount()>0 is a workaround for a bug in netscape4-java
      if(refL.getItemCount()>0) refL.removeAll(); 
      LinkTag ut;
    
      synchronized(vector)
      {
        int i=list.getSelectedIndex();
        if(i<0) return;
        ut=(LinkTag)(vector.elementAt(i));
      }
    
      int num=0;
      for(Enumeration e=ut.getReferrers(); e.hasMoreElements(); )
      {
        if((num++)<20) 
          refL.add(e.nextElement().toString());
        else
        {
          refL.add(".. and more");
          break;
        }
      }
    }  
  } // class LinkFrame.Chart
  

  protected Chart chartUnsorted;
  protected Chart chartUrls;
  protected Chart chartErrors;
  protected Chart chartTargets;
  protected Progress progressFrame;
  
  /**
    Constructor for the main window of LinkVerify.
    @param root the url which is initially the <B>root url</B>
    @param verifyExternals the initial value of the corresponding checkbox
    @param nthreads the number of checker spiders working paralelly
  */          
  public LinkResult(java.net.URL root,
                    boolean externals,
                    int nthreads)
  {
    super("LinkVerify - "+root.toString());
    setSize(600,500);
    this.root=root;
    this.externals=externals;
    this.nthreads=nthreads;

    setLayout(new BorderLayout(10,10));
    Panel buttonP;
        
    {
      Panel statusP=new Panel(new FlowLayout(FlowLayout.RIGHT));
      statusP.add(progress=new Button("Progress"));
      statusP.add(abort=new Button("Abort"));
      statusP.add(close=new Button("Close"));
      add(statusP, BorderLayout.SOUTH);
    }
    {
      Panel P=new Panel(new BorderLayout());
      {
        Panel Q=new Panel(new BorderLayout());
        Q.add(new Label("Results"), BorderLayout.NORTH);
        Q.add(buttonP=new Panel(new GridLayout(1,0)), BorderLayout.SOUTH);
        P.add(Q, BorderLayout.NORTH);
      }
      {
        Panel resultsP=new Panel(new GridLayout(2,1));
        {
          Panel chartP=new Panel(chartL=new CardLayout());
          chartCt=chartP;
          chartUnsorted=new Chart(chartP, buttonP, "Unsorted", "Dead links (relative to root url) unsorted");
          chartUrls    =new Chart(chartP, buttonP, "Urls",     "Dead links (relative to root url) sorted by url");
          chartErrors  =new Chart(chartP, buttonP, "Errors",   "Dead links (relative to root url) sorted by error");
          chartTargets =new Chart(chartP, buttonP, "Frames",   "Frame names encountered");
          resultsP.add(chartP);
        }
        {
          Panel referreeP=new Panel(new BorderLayout());
          referreeP.add(new Label("Referrers (relative to root url)"), BorderLayout.NORTH);
          referreeP.add(refL=new List(), BorderLayout.CENTER);
          resultsP.add(referreeP);
        }
        P.add(resultsP, BorderLayout.CENTER);
      }
      add(P, BorderLayout.CENTER);
    }

    progressFrame=new Progress(nthreads);

    abort.addActionListener(this);
    progress.addActionListener(this);
    close.addActionListener(this);
    this.addWindowListener(this);
    System.out.println("root("+root.getProtocol()+"|"+root.getHost()+"|"+root.getPort()+"|"+root.getFile()+"|"+root.getRef()+")");
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource()==abort)
    {
      abort();
    }
    else if(e.getSource()==close)
    {
      abort(); progressFrame.dispose(); dispose();
    }
    else if(e.getSource()==progress)
    {
      if(progressFrame.isVisible())
        progressFrame.hide();
      else
        progressFrame.show();
    }
    else
    {
      throw new IllegalArgumentException();
    }
  }

  public void start()
  {
    synchronized(chartUnsorted)
    {
      //System.err.println("LinkResult.start()");
      if(spider!=null) throw new IllegalArgumentException();

      chartUnsorted.reset();
      chartUrls.reset();
      chartErrors.reset();
      chartTargets.reset();

      // the getItemCount()>0 is a workaround for a bug in netscape4-java
      if(refL.getItemCount()>0) refL.removeAll();

      spider=new LinkSpider(this, root, externals, nthreads);
      spider.startVerify();
    }
  }

  public void abort()
  {
    synchronized(chartUnsorted)
    {
      //System.err.println("stop");
      if(spider!=null)
      {
        spider.stop();
        spider=null;
        progressFrame.hide();
        for(int i=0; i<nthreads; i++) progressFrame.st[i].setText("aborted.");
      }
      //System.err.println("/LinkVerify.startstop()");
    }
  }
  
  public void onEnd()
  {
    spider=null;
    progressFrame.hide();
    for(int i=0; i<nthreads; i++) progressFrame.st[i].setText("done.");
  }
 
  public void onLookup(LinkTag tag, int thread)
  {
    progressFrame.st[thread].setText(tag.toString());
  }

  public void onWaiting(int thread)
  {
    progressFrame.st[thread].setText("waiting...");
  }
  
  public static int sortIntoList(List L, String s)
  {
    for(int i=0; i<L.getItemCount(); i++)
      if(s.compareTo(L.getItem(i))<0)
      {
        L.addItem(s, i);
        return i;
      }
    L.addItem(s);
    return -1;
  }

  public void onAbsent(LinkTag tag)
  {
    String T=tag.toString();
    String E=tag.getError();
    String formatTE=format(T,E);
    String formatET=format(E,T);
    Chart chart;

    synchronized(chart=chartUnsorted)
    {
      chart.list.add(formatTE);
      chart.vector.addElement(tag);
    }
    synchronized(chart=chartUrls)
    {
      int i=sortIntoList(chart.list, formatTE);
      if(i>=0) chart.vector.insertElementAt(tag, i);
          else chart.vector.addElement(tag);
    }
    synchronized(chart=chartErrors)
    {
      int i=sortIntoList(chart.list, formatET);
      if(i>=0) chart.vector.insertElementAt(tag, i);
          else chart.vector.addElement(tag);
    }
  }
  
  public void onTarget(LinkTarget target)
  {
    Chart chart=chartTargets;
    int num=target.getNumber();
    synchronized(chart)
    {
      if(num<chart.list.getItemCount())
      {
        chart.list.replaceItem(target.toString(),num);
        chart.vector.setElementAt(target,num);
      }
      else 
      {
        chart.list.addItem(target.toString());
        chart.vector.addElement(target);
      }
    }
  }
  
  public void windowActivated(WindowEvent e){}
  public void windowDeactivated(WindowEvent e){}
  public void windowDeiconified(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowOpened(WindowEvent e){start();}
  public void windowClosing(WindowEvent e){abort();progressFrame.dispose();dispose();}
  public void windowClosed(WindowEvent e){}

  public final static String format(String url, String err)
  {
    StringBuffer s=new StringBuffer();
    s.append(url);
        
    int l=30-s.length();
    while(l<=5)l+=10;
    
    for(int i=0; i<l; i++) s.append(' ');
    s.append(err);
    return s.toString();
  }

  class Progress
    extends Frame
    implements WindowListener,
               ActionListener
  {
    /**
      The TextCommonents, where each spider thread shows
      the url currently checked.
    */
    protected TextField[] st;

    public Progress(int nthreads)
    {
      super("LinkVerify Progress");
      setLayout(new BorderLayout());

      Button close;
      add(close=new Button("Close"),BorderLayout.SOUTH);

      {
        Panel P=new Panel(new GridLayout(0,1));
        st=new TextField[nthreads];
        for(int i=0; i<nthreads; i++)
        {
          P.add(st[i]=new TextField());
          st[i].setEditable(false);
        }
        add(P, BorderLayout.NORTH);
      }

      setSize(200,200);
      addWindowListener(this);
      close.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e){ hide(); }
    public void windowActivated(WindowEvent e){}
    public void windowDeactivated(WindowEvent e){}
    public void windowDeiconified(WindowEvent e){}
    public void windowIconified(WindowEvent e){}
    public void windowOpened(WindowEvent e){}
    public void windowClosing(WindowEvent e){ hide(); }
    public void windowClosed(WindowEvent e){}
  }

}

