/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.htmltools;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.filechooser.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.File;

public class LinkSwing 
  extends JFrame 
  implements LinkFrontend,
             WindowListener,
             ActionListener
{
  
  private static final String[] columnNamesUrls={"url","error"};
  private static final String[] columnNamesTargets={"name","defined"};
  
  private static final int TIMER_RATE_ACTIVE = 500;
  private static final int TIMER_RATE_INACTIVE = 1000;
    
  private class LinkM extends AbstractTableModel
  {
    
    public int getColumnCount()
    {
      return columnNamesUrls.length;
    }
    
    public String getColumnName(int column)
    {
      return columnNamesUrls[column];
    }
  
    private Vector data=new Vector();
    
    public int getRowCount()
    {
      return data.size();
    }
  
    public Object getValueAt(int row, int column)
    {
      Object tag=data.elementAt(row);
      return (column==0) ? tag : ((LinkTag)tag).getError();
    }
    
    private void appendData(LinkTag tag)
    {
      int row=data.size();
      data.addElement(tag);
      fireTableRowsInserted(row,row);
    }

    private void clear()
    {
      int row=data.size();
      if(row>0)
      {
        data.setSize(0);
        fireTableRowsDeleted(0,row-1);
      }
    }
  }

  class FrameM extends AbstractTableModel
  {
    public int getColumnCount()
    {
      return columnNamesTargets.length;
    }
    
    public String getColumnName(int column)
    {
      return columnNamesTargets[column];
    }
    
    private Vector data=new Vector();
  
    public int getRowCount()
    {
      return data.size();
    }
  
    public Object getValueAt(int row, int column)
    {
      LinkTarget target=(LinkTarget)(data.elementAt(row));
      switch(column)
      {
      case 0:  return target.getName();
      default: Object o=target.getDefiner();
               return (o!=null) ? o.toString() : "UNDEFINED";
      }
    }
    
    private void appendData(LinkTarget target)
    {
      int row=data.size();
      int num=target.getNumber();
      synchronized(data)
      {
        if(num<row) data.setElementAt(target,num);
               else data.addElement(target);
      }
      if(num<row) fireTableRowsUpdated(num,num);
             else fireTableRowsInserted(num,num);
    }

    private void clear()
    {
      int row=data.size();
      if(row>0)
      {
        data.setSize(0);
        fireTableRowsDeleted(0,row-1);
      }
    }
    
  }

  private class TabL implements MouseListener, ActionListener
  {
    private JTable tab;
    private Hashtable ht=new Hashtable();
    
    TabL(JTable tab)
    {
      this.tab=tab;
      tab.addMouseListener(this);
    }
    
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) 
    {
      if(e.getClickCount()!=1) return;
      //System.err.println(e);
      Point p=e.getPoint();
      //System.err.println(p);
      int row=tab.rowAtPoint(p);
      int col=tab.columnAtPoint(p);
      //System.err.println("Table("+row+","+col+")");
      if(row>=0&&col>=0)
      {
        Object err=tab.getValueAt(row, col);
        if(err!=null&&err instanceof LinkTag)
        {
          //System.err.println(o);
          LinkTag tag=(LinkTag)err;
          
          ht.clear();
          JPopupMenu pu=new JPopupMenu();
          for(Enumeration iref=tag.getReferrers();  iref.hasMoreElements(); )
          {
            Object ref=iref.nextElement();
            String sref=ref.toString();
            JMenuItem mi=new JMenuItem(sref);
            mi.addActionListener(this);
            pu.add(mi);
            ht.put(sref,ref);
          }  
          pu.show(e.getComponent(), e.getX(), e.getY());
        }
      }
    }

    public void actionPerformed(ActionEvent e)
    {
      //System.err.println(e.getActionCommand());
      Object o=ht.get(e.getActionCommand());
      if(o!=null&&o instanceof LinkFile)
      {
        Editor editor=new Editor((LinkFile)o);
        editor.show();
      }

/*
          try
          {
            (new JEditorPane(((LinkFile)o).url)).show();
          }
          catch(java.io.IOException ex){System.err.println(ex);}
*/
    }

  }

  protected LinkSpider spider;
  protected LinkM linkM;
  protected FrameM frameM;
  
  protected JTextField[] st;
  protected JTextField rootTF;
  protected JCheckBox verifyCB;
  protected JToggleButton startBT;
  protected Timer memoryTimer;

  public LinkSwing(String root,
                   boolean verifyExternals,
                   int nthreads)
  {
    super("LinkVerify");
    setSize(400,400);

    java.awt.Container cp=getContentPane();
    cp.setLayout(new BorderLayout());
        
    JPanel ctrl=new JPanel(new GridLayout(3,1));
    cp.add(ctrl, BorderLayout.NORTH);
    {
      JPanel rootP=new JPanel(new BorderLayout());
      JButton rootBT=new JButton("..");
      rootBT.addActionListener(this);
      rootBT.setToolTipText("choose a local file");
      rootP.add(rootBT, BorderLayout.EAST);
      rootTF=new JTextField(root);
      rootTF.setToolTipText("the spider starts at this url");
      rootP.add(rootTF, BorderLayout.CENTER);
      ctrl.add(rootP);
      
      verifyCB=new JCheckBox("Verify external links", verifyExternals);
      verifyCB.setToolTipText("verify links outside the dir/subdirs of root url");
      ctrl.add(verifyCB, BorderLayout.CENTER);
    
      startBT=new JToggleButton("start");
      startBT.addActionListener(this);
      startBT.setToolTipText("start the verification process");
      ctrl.add(startBT);
    }
    
    JTabbedPane tb=new JTabbedPane();
    cp.add(tb, BorderLayout.CENTER);
    {
      {
        JTable lt=new JTable(linkM=new LinkM());
        tb.addTab("Links", null, new JScrollPane(lt), "shows invalid links");
        new TabL(lt);
      }
      {
        JTable ft=new JTable(frameM=new FrameM());
        tb.addTab("Frames", null, new JScrollPane(ft), "shows frame names");
        new TabL(ft);
      }
      
      Box pp=Box.createVerticalBox();
      tb.addTab("Progress", null, pp, "show the progress of the verification process");
      {
        st=new JTextField[nthreads];
        for(int i=0; i<nthreads; i++)
        {
          st[i]=new JTextField();
          st[i].setToolTipText("the state of verification thread #"+(i+1));
          pp.add(st[i]);
        }
        MemoryModel memmod=new MemoryModel();
        JProgressBar pb=new JProgressBar(memmod);
        pb.setStringPainted(true);
        JLabel label=new JLabel("Memory Usage");
        pp.add(label);
        pp.add(pb);
        memoryTimer=new Timer(TIMER_RATE_INACTIVE, memmod);
        memoryTimer.start();
      }
    }
    addWindowListener(this);
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource()==startBT)
    {
      startstop();
    }
    else
    {
      JFileChooser chooser=new JFileChooser();
      
      FileFilter filter=
        new FileFilter()
        {
          public boolean accept(File file)
          {
            if(file==null||file.isDirectory() ) return true;
            String s=file.getName();
            return
              s==null ||
              s.endsWith(".html") ||
              s.endsWith(".shtml") ||
              s.endsWith(".htm") ||
              s.endsWith(".css");
          }
          
          public String getDescription()
          {
            return "Hypertext (html,shtml,htm,css)";
          }
        };

      chooser.addChoosableFileFilter(filter);
      chooser.setFileFilter(filter);

      if(chooser.showDialog(this, null)==JFileChooser.APPROVE_OPTION)
      {
        File file=chooser.getSelectedFile();
        if(file!=null&&!file.isDirectory())
          rootTF.setText("file:///"+file.getAbsolutePath());
      }
    }
  }
  
  public void startstop()
  {
    //System.err.println("LinkVerify.startstop()");
    if(spider==null)
    {
      //System.err.println("start");
      java.net.URL u;
      try
      {
        u=LinkVerify.convert(rootTF.getText());
      }
      catch(java.net.MalformedURLException ex)
      {
        (new MessageDialog(this, "Invalid Root URL","The URL you have entered\nis not valid.\n\n"+ex.getMessage())).show();
        return;
      }

      linkM.clear();
      frameM.clear();
      startBT.setText("stop");
      spider=new LinkSpider(this,
                            u,
                            verifyCB.isSelected(), 
                            st.length);
      spider.startVerify();
      memoryTimer.setDelay(TIMER_RATE_ACTIVE);
    }
    else
    {
      //System.err.println("stop");
      spider.stop();
      spider=null;
      startBT.getModel().setPressed(false);
      startBT.setText("start");
      for(int i=0; i<st.length; i++) st[i].setText("done.");
      memoryTimer.setDelay(TIMER_RATE_INACTIVE);
    }
    //System.err.println("/LinkVerify.startstop()");
  }

  public void onEnd()
  {
      spider=null;
      startBT.getModel().setPressed(false);
      startBT.setText("start");
      for(int i=0; i<st.length; i++) st[i].setText("done.");
      memoryTimer.setDelay(TIMER_RATE_INACTIVE);
  }
 
  public void onLookup(LinkTag tag, int thread)
  {
    //System.err.println("lookup "+tag+thread);
    st[thread].setText(tag.toString());
  }

  public void onWaiting(int thread)
  {
    //System.err.println("waiting..."+thread);
    st[thread].setText("waiting...");
  }
  
  public void onAbsent(LinkTag tag)
  {
    //System.err.println(tag.toString());
    linkM.appendData(tag);
  }
    
  public void onTarget(LinkTarget target)
  {
    //System.err.println(target.toString());
    frameM.appendData(target);
  }

  public void windowActivated(WindowEvent e){}
  public void windowDeactivated(WindowEvent e){}
  public void windowDeiconified(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowOpened(WindowEvent e){}
  public void windowClosing(WindowEvent e){dispose();}
  
  public boolean exitJVMonDispose=false;
  public void windowClosed(WindowEvent e)
  {
    if(exitJVMonDispose) System.exit(0);
  }
  
  public static void main(String[] args)
  {
    LinkSwing f=new LinkSwing("file:///home/rw7/public_html/htmltools/index.html",false,4);
    f.exitJVMonDispose=true;
    f.show();
  }

}
