/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.htmltools;

import java.util.Enumeration;
import java.util.Vector;
import java.net.URL;
import java.net.MalformedURLException;

public class LinkVerify implements LinkFrontend
{

  LinkSpider spider;

  public void runVerify(String root, 
                        boolean externals,
                        int nthreads)
  {
    root=LinkTag.replaceBackslashes(root);
    try
    {
      java.net.URL u=new java.net.URL(root);
      spider=new LinkSpider(this, u, externals, nthreads);
      spider.startVerify();
    }
    catch(java.net.MalformedURLException ex)
    {
      System.out.println("Invalid Root URL: "+ex.toString());
    }
  }
  
  public void onLookup(LinkTag tag, int thread) {}

  public void onWaiting(int thread) {}
  
  synchronized public void onAbsent(LinkTag tag)
  {
    System.out.println(tag.toString());
    System.out.print("    ");
    System.out.println(tag.getError());
    for(Enumeration e=tag.getReferrers(); e.hasMoreElements(); )
    {
      System.out.print("    ");
      System.out.println(e.nextElement().toString());
    }
  }
  
  private Vector targets=new Vector();
  
  public void onTarget(LinkTarget target)
  {
    int number=target.getNumber();
    if(targets.size()<=number) targets.setSize(number+1);
    targets.setElementAt(target, number);
  }
  
  public void onEnd()
  {
    spider=null;
    if(targets.size()>0)
    {
      System.out.println("following targets found:");    
      for(int i=0; i<targets.size(); i++)
      {
        LinkTarget target=(LinkTarget)(targets.elementAt(i));
        System.out.println(target.toString());
        for(Enumeration e=target.getReferrers(); e.hasMoreElements(); )
        {
          System.out.print("    ");
          System.out.println(e.nextElement().toString());
        }
      }
    }
  }
  
  
  /**
    The public main method to invoke the LinkVerifier from
    command line. If invoked this way, the LinkVerifier will 
    exit the JVM after closing the main window.
  */    
  public static void main(String[] args)
  {
    boolean text=false;
    String root=null;
    boolean externals=true;
    int nthreads=4;
    
    for(int i=0; i<args.length; i++)
    {
           if("-x".equals(args[i])||
              "--noexternals".equals(args[i])) externals=false;
      else if("-c".equals(args[i])||
              "--console".equals(args[i])) text=true;
      else if(root==null) root=args[i];
      else 
      {
        System.out.println("usage: java wiebicke.htmltools.LinkVerify [-x] [-c] [root url]");
	return;
      }
    }
    
    if(text&&root!=null)
    {
      LinkVerify lv=new LinkVerify();
      lv.runVerify(root,externals,nthreads);
    }
    else
    {
      LinkFrame lf=new LinkFrame(root,externals,nthreads);
      lf.exitJVMonDispose=true;
      lf.show();
    }
  }


  /**
    Parses an incomplete URL. Used for user-typed URLS only.
  */
  public static URL convert(String s) throws MalformedURLException
  {
    URL u;
    s=LinkTag.replaceBackslashes(s);
    try
    {
      return new URL(s);
    }
    catch(MalformedURLException ex)
    {
      int colon=s.indexOf((int)':'); if(colon<0) colon=s.length();
      int slash=s.indexOf((int)'/'); if(slash<0) slash=s.length();
      int dot  =s.indexOf((int)'.'); if(dot  <0) dot  =s.length();
      if(colon<slash&&colon<dot)
      {
        // there is either an unknown protocol or a Windows-Drive-Letter
        if(colon==1)
        {
          // it is a Windows-Drive-Letter
          try
          {
            return new URL("file:"+s);
          }
          catch(MalformedURLException e) { throw ex; }
        }
        else throw ex;
      }
      else if(dot<slash)
      {
        // it is an http-URL lacking the prefix "http://"
        try
        {
          return new URL("http://"+s);
        }
        catch(MalformedURLException e) { throw ex; }
      }
      else
      {
        // it is an file-URL without a Windows-Drive-Letter lacking the prefix "file:/"
        try
        {
          return new URL("file:"+s);
        }
        catch(MalformedURLException e) { throw ex; }
      }
    }
  }
  
}
