/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.htmltools.test;

import de.rw7.htmltools.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
  Relies on consistent enumeration order of hashtable!
*/
public class SpiderTest implements LinkFrontend
{

  LinkSpider spider;
  int nThreads=4;

  public void runVerify()
  {
    URL root=SpiderTest.class.getResource("/de/rw7/htmltools/test/test.html");
    spider=new LinkSpider(this,root,true,nThreads);
    spider.startVerify();
  }

  public void onLookup(LinkTag tag, int thread)
  {
    if(thread<0||thread>=nThreads) System.out.println("OOOPS");
    System.out.println("onLookup:-"+tag+'-'+thread);
  }

  public void onWaiting(int thread)
  {
    if(thread<0||thread>=nThreads) System.out.println("OOOPS2");
    System.out.println("onWaiting:"+thread);
  }

  synchronized public void onAbsent(LinkTag tag)
  {
    System.out.println("onAbsent:-"+tag+'-');
  }

  public void onTarget(LinkTarget target)
  {
    System.out.println("onTarget:"+target);
  }

  private void alarm(String s)
  {
    System.out.println(s);
  }

  public void onEnd()
  {
    try
    {
      PrintStream o=new PrintStream(new FileOutputStream("de/rw7/htmltools/test/SpiderTest.log"));
      o.println("Generated by SpiderTest.class");

      Hashtable pool=spider.getPool();
      o.println("POOL: size "+pool.size());
      for(Enumeration uE=pool.keys(); uE.hasMoreElements(); )
      {
        URL u=(URL)uE.nextElement();
        LinkFile lf=(LinkFile)pool.get(u);
        o.println(u.toString()+'-'+lf+'-'+getStateString(lf.getState()));
        if(lf.getState()!=2)
        {
          o.println("  REFERRERS");
          for(Enumeration uR=lf.getReferrers(); uR.hasMoreElements(); )
          {
            LinkFile ref=(LinkFile)uR.nextElement();
            o.println("    -"+ref+'-'+getStateString(ref.getState()));
          }
        }
        o.println("  ANCHORS");
        Hashtable anchors=lf.getAnchors();
        if(anchors==null)
          o.println("    null");
        else
        {
          for(Enumeration aE=anchors.keys(); aE.hasMoreElements(); )
          {
            String s=(String)aE.nextElement();
            LinkAnchor an=(LinkAnchor)anchors.get(s);
            o.println("   -"+s+'-'+an+'-'+(an.isFound()?"FOUND":"NOTFOUND"));
            if(!an.isFound())
            {
              o.println("      ANCHOR-REFERRERS");
              for(Enumeration uR=an.getReferrers(); uR.hasMoreElements(); )
              {
                LinkFile ref=(LinkFile)uR.nextElement();
                o.println("        -"+ref+'-'+getStateString(ref.getState()));
              }
            }
          }
        }
      }
      o.println("VAKANT:");
      for(Enumeration vE=spider.getVakant(); vE.hasMoreElements(); )
      {
        LinkFile lf=(LinkFile)vE.nextElement();
        o.println("  -"+lf+'-'+getStateString(lf.getState()));
      }
      Hashtable targets=spider.getTargets();
      o.println("TARGETS:");
      for(Enumeration tE=targets.keys(); tE.hasMoreElements(); )
      {
        String s=(String)tE.nextElement();
        LinkTarget tgt=(LinkTarget)targets.get(s);
        o.println(" -"+s+'-'+tgt);
      }
    }
    catch(Exception e){System.err.println(e);e.printStackTrace(System.err);};
  }

  public static void main(String[] args)
  {
    (new SpiderTest()).runVerify();
  }

  public static final String getStateString(int state)
  {
    switch(state)
    {
    case 0: return "VAKANT";
    case 1: return "LOOKUP";
    case 2: return "FOUND";
    case 3: return "ABSENT";
    default: throw new IllegalArgumentException();
    }
  }

}
