/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.token;

import java.io.*;
import java.util.*;

public class TextTokenizer
{

  private StringBuffer txt=new StringBuffer(300);
  private StringBuffer ent=new StringBuffer(30);
  
  public final String getText()
  {
    if(txt.length()>0)
    {
      if(ws) { txt.append(' '); ws=false; }
      String s=new String(txt);
      txt.setLength(0);
      return s;
    }
    else
      return null;
  }
  
  public final String peekText()
  {
    if(txt.length()>0)
    {
      if(ws) return new String(txt)+' ';
        else return new String(txt);
    }
    else
      return null;
  }

  public boolean ignoreEntitys=false;
  
  /**
    Indicates that there is there a pending whitespace.
  */
  private boolean ws=false;
    
  public final void read(FastInput in) throws IOException, FastInput.EndException
  {
    char buf=in.read();
    
    textL: while(buf!='<')
    { 
    
      if(buf==' '||buf=='\t'||buf=='\n'||buf=='\r')
      {
        ws=true;
        buf=in.read();
        continue textL;
      }

      if(buf!='&' || ignoreEntitys)
      {
        if(ws) { txt.append(' '); ws=false; }
        txt.append(buf);
        buf=in.read();
        continue textL;
      }

      buf=in.read();
      
      while(buf!=';')
      {
        ent.append(buf);
        buf=in.read();
      }
 
      String s=new String(ent); ent.setLength(0);
        
      char c=getConverted(s);
      if(ws) { txt.append(' '); ws=false; }
      if(c=='\0') 
      { 
        txt.append('&');
        txt.append(s);
        txt.append(';');
      }
      else
        txt.append(c);
        
      buf=in.read();
           
    } // textL

    in.pushBack();
  }
  
  static private final char[] chr= 
  {
    '&','<','>','\"',
    '','','','','','','','',
    ' '
  };
  static private final String[] str=
  {
    "amp","lt","gt","quot",
    "Auml","auml","Ouml","ouml","Uuml","uuml","szlig","eacute",
    "nbsp"
  };
  
  static private final int htmlLen=4, formLen=1;

  static public final String toHtmlHtml(String x)
  {
    return toHtml(x, 0, htmlLen);
  }

  static public final String toHtmlChar(String x)
  {
    return toHtml(x, htmlLen, chr.length-formLen);
  }

  static public final String toHtmlAll(String x)
  {
    return toHtml(x, 0, chr.length-formLen);
  }

  /**
  Konvertiert einen String mit Nicht-Ascii-Zeichen in eine
  HTML-konforme Zeichenkette. Nur eine Teil der Zeichen wird unterst&uuml;tzt.
  */
  static private final String toHtml(String x, int start, int end)
  {
    int j;
    for(int i=start; i<end; i++)
      if((j=x.lastIndexOf(chr[i]))>=0)
      {
        StringBuffer sb=new StringBuffer(x);
        do
        {
          sb.setCharAt(j,';');
          sb.insert(j,str[i]);
          sb.insert(j,'&');
        }
        while((j=x.lastIndexOf(chr[i],j-1))>=0);

        x=new String(sb);
      }
    return(x);
  }
  
  private static java.util.Hashtable ht;
  /**
  Konvertiert einen Html-Eintity-String in eine Zeichen. Wenn der
  String nicht konvertiert werden kann, wird '\0' zu&uuml;ckgegeben.
  */
  public static char getConverted(String s)
  {
    if(ht==null)
    {
      ht=new java.util.Hashtable();
      for(int i=0; i<chr.length; i++) 
        ht.put(str[i], new Character(chr[i]));
    }
    Character c=(Character)(ht.get(s));
    return (c==null) ? '\0' : c.charValue();
  }
  
}
  
    
