/*
  Copyright (C) 1998 Ralf Wiebicke

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You'll find the GPL at http://www.gnu.org/copyleft/gpl.html
*/

package de.rw7.token.test;

import de.rw7.token.*;
import java.io.*;
import java.util.*;

public class TokenTest implements HtmlConsumer, Runnable
{
  int error=0;
  HtmlTokenizer ht;
  TextTokenizer tt;
  FastInput r;

  Vector tagV=new Vector();
  int tagI=0;  

  class tag implements HtmlConsumer
  {
    char type;
    int code;
    String name;
    String[] attr;
    int response;
    Runnable action;
    
    tag(int Acode, String Aname, String[] Aattr, int Aresponse, Runnable Aaction)
    {
      type='O';
      code=Acode;
      name=Aname;
      attr=Aattr;
      response=Aresponse;
      action=Aaction;
    }  
    public int onOpeningTag(int Acode, String Aname, String[] Aattr)
    {
      tagI++;
      if(type!='O') alarm('O',Aname,"type");
      if(code!=Acode) alarm('O',Aname,"code("+code+'|'+Acode+')');
      if(name!=Aname) alarm('O',Aname,"name"); // do not have to use equals()
      if(attr==null&&Aattr==null)
        ;
      else if(attr!=null&&Aattr!=null)
      {
        if(attr.length!=Aattr.length) alarm('O',Aname,"attr.length");
        for(int i=0; i<attr.length; i++)
          if((attr[i]==null&&Aattr[i]==null)||
            (attr[i]!=null&&attr[i].equals(Aattr[i])))
             ;
          else 
            alarm('O',Aname,"attr("+i+'|'+attr[i]+'|'+Aattr[i]+')');
      }     
      else
        alarm('O',name,"attr==null");

      if(action!=null) action.run();
      System.out.println("O:-"+Aname+'-');
      return response;
    }  

    tag(int Acode, String Aname, int Aresponse)
    {
      type='C';
      code=Acode;
      name=Aname;
      attr=null;
      response=Aresponse;
    }
    public int onClosingTag(int Acode, String Aname)
    {
      tagI++;
      if(type!='C') alarm('C',Aname,"type");
      if(code!=Acode) alarm('C',Aname,"code("+code+'|'+Acode+')');
      if(name!=Aname) alarm('C',Aname,"name");
      if(attr!=null) alarm('C',Aname);
      System.out.println("C:-"+Aname+'-');
      return response;
    }
    
    tag(String Aname)
    {
      type='T';
      name=Aname;
    }
    public void onText(FastInput in) throws IOException, FastInput.EndException
    {
      tt.read(in);
      if(type=='T')
      {
        String s=tt.peekText();
        if(s!=null)
        {
          if(name.equals(s)) 
          { 
            String sx=tt.getText(); 
            System.out.println("T:-"+sx+'-');
            tagI++; 
          }
          else if(!name.startsWith(s)) alarm('T',s);
        }  
      }  
    }  
      
    private final void alarm(char c, String s, String desc)
    {
      System.out.println("ERR("+(error++)+"): ("+c+")-"+convert(s)+"- at ("+type+")-"+name+'-'+desc+r.toString());
    }

    private final void alarm(char c, String s)
    {
      alarm(c, s, null);
    }
    
  } // inner class tag  
      
  private final void alarm(String s)
  {
    System.out.println("ERr("+(error++)+"): -"+convert(s)+'-');
  }
    
  private static final String convert(String s)
  {
    if(s.indexOf((int)'\n')<0)
      return s;
    else  
    {
      StringBuffer b=new StringBuffer(s);
      int pos=-1;
      while((pos=s.indexOf((int)'\n',pos+1))>=0)
      b.setCharAt(pos,'\\');
      return new String(b);
    }
  }
  
  public int onOpeningTag(int code, String name, String[] attr)
  {
    return ((tag)(tagV.elementAt(tagI))).onOpeningTag(code, name, attr);
  }  
  
  public int onClosingTag(int code, String name)
  {
    return ((tag)(tagV.elementAt(tagI))).onClosingTag(code, name);
  }
  
  public void onText(FastInput in) throws IOException, FastInput.EndException
  {
    ((tag)(tagV.elementAt(tagI))).onText(in);
  }
  
  public void run()
  {
    HtmlTokenizer.test();
      
    int mode=HtmlTokenizer.MODE_PARSE_TEXT|HtmlTokenizer.MODE_STRICT_TAGS|HtmlTokenizer.MODE_STRICT_ATTR;
    ht=new HtmlTokenizer(mode);
    tt=new TextTokenizer();
        
    String sTxt="T& Xt";
    String[] sKey={ "SKEY", "PKEY" };
    String[] sVal={ HtmlTokenizer.PRESENT, "pVaL" };
    String[] sVal2={ HtmlTokenizer.PRESENT, "pVaL pVaL" };
      
    ht.addOpeningTag("SWITCH",'0', null );
    ht.addOpeningTag("TAGL09AZ",'L', null );
    ht.addOpeningTag("TAGO09AZ", 'O', sKey );
    ht.addClosingTag("TAGC09AZ", 'C');
    ht.addTag("TAGXAZ09001", 'X', null);
    ht.removeTag("TAGXAZ09001");
    ht.printTree(System.out);
        
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('L',"TAGL09AZ",null,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('L',"TAGL09AZ",null,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('L',"TAGL09AZ",null,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('C',"TAGC09AZ",mode));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('C',"TAGC09AZ",mode));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('C',"TAGC09AZ",mode));

    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal2,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    tagV.addElement(new tag(sTxt));
    tagV.addElement(new tag('O',"TAGO09AZ",sVal,mode,null));
    
    mode=HtmlTokenizer.MODE_PARSE_TEXT;
    tagV.addElement(new tag('0',"SWITCH",null,mode,
    new Runnable()
    {
      public void run()
      {
        ht.removeOpeningTag("TAGO09AZ");
        ht.removeClosingTag("TAGC09AZ");
        System.out.println("SWITCH 2");
        ht.printTree(System.out);
      }
    }  
    ));
    String tTxt=" Das ist der gesuchte Satz.";
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));

    mode=HtmlTokenizer.MODE_PARSE_TEXT|HtmlTokenizer.MODE_IGNORE_COMMENTS;
    tagV.addElement(new tag('0',"SWITCH",null,mode,null));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    tagV.addElement(new tag(tTxt));
    
    mode=HtmlTokenizer.MODE_PARSE_TEXT|HtmlTokenizer.MODE_STRICT_TAGS;
    String[] kVal={ "knownval","knownval1","knownval2" };
    tagV.addElement(new tag('0',"SWITCH",null,mode,
    new Runnable()
    {
      public void run()
      {
        String[] kKey={ "KNOWNATTR","KNOWNATTR1","KNOWNATTR2" };
        ht.addOpeningTag("KNOWNTAG",'K',kKey);
        System.out.println("SWITCH 4");
        ht.printTree(System.out);
      }
    }  
    ));
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    
    tagV.addElement(new tag('K',"KNOWNTAG",kVal,mode,null));    

    mode=0;
    tagV.addElement(new tag('0',"SWITCH",null,mode,null));

    try
    {
      r=new FastInput();
      r.open(TokenTest.class.getResourceAsStream("/de/rw7/token/test/test.html"));
      try { ht.read(r, this); } catch(FastInput.EndException e){};
      r.close();
    }
    catch(IOException e) { System.out.println(e); return; }
    if(tagI!=tagV.size()) alarm("size("+tagI+'|'+tagV.size()+')'+'-'+tt.getText()+'-');
    ht.removeOpeningTag("SWITCH");
    
    ht.addTag("TAGO09AZ",'O',null);
    ht.addTag("TAGC09AZ",'C',null);
    ht.addTag("tag",0,null);
    ht.addTag("tagx",1,null);
    ht.addTag("tagO09AZwurm",2,null);
    ht.printTree(System.out);

    try 
    {
      ht.addTag("TAGO09AZ",'O',null);
      alarm("ungueltiges add() passiert");
    }
    catch (IllegalArgumentException e) { System.out.println("OK: "+e); }

    try
    {
      ht.removeTag("TAGX0");
      alarm("ungueltiges remove() passiert");
    }
    catch (IllegalArgumentException e) { System.out.println("OK: "+e); }

    try
    {
      ht.removeTag("TAGX00012");
      alarm("ungueltiges remove() passiert");
    }
    catch (IllegalArgumentException e) { System.out.println("OK: "+e); }
        
    if(error!=0) System.out.println("ERRORS OCCURED: "+error);
            else System.out.println("no errors occured");
  }
  
  public static void main(String[] args) throws IOException
  {
    (new TokenTest()).run();
  }  
    
}
